import React, { Component } from 'react'
import PropTypes from 'prop-types'
import { BubbleSidebarActiveItemContext } from './context'
import { Header, Item } from './components'
import * as C from './constants'
import { concatClassNamesByCondition } from './helper'
import './BubbleSidebar.scss'

class BubbleSidebar extends Component {
  static defaultProps = {
    attached: false,
    width: 'normal',
    color: '#fff',
    background: '#000',
    sliderBackground: '#fff',
    activeIndex: 0,
    activeColor: '#000'
  }

  constructor(props) {
    super(props)

    this.state = {
      background: 'transparent',
      activeIndex: props.activeIndex,
      activeColor: props.activeColor,
      makeItemActive: this.makeItemActive
    }

    this.sidebar = React.createRef()
    this.slider = React.createRef()
    this.moving = false
  }

  componentDidMount() {
    this.activeItemInfirstRender()
  }

  makeItemActive = index => {
    return () => {
      const prevActiveIndex = this.state.activeIndex
      if (this.moving || index === prevActiveIndex) {
        return
      }

      this.setState(
        {
          background: this.props.background,
          activeIndex: index
        },
        () => {
          this.effectMoveSlider(prevActiveIndex)
        }
      )
    }
  }

  activeItemInfirstRender = () => {
    const sidebarElement = this.sidebar.current
    const sliderElement = this.slider.current

    const activeIndex = this.state.activeIndex
    const sliderTo = sidebarElement.children[activeIndex + 1]
    const moveLength = Array.from(sidebarElement.children)
      .slice(1)
      .slice(0, activeIndex)
      .reduce((acc, child) => {
        return acc + child.offsetHeight
      }, 0)

    if (activeIndex === 0) {
      const item = sidebarElement.children[2]

      switch (this.props.width) {
        case 'thin':
          item.style.borderTopRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
          break
        case 'normal':
          item.style.borderTopRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
          break
        case 'wide':
          item.style.borderTopRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
          break
        default:
          break
      }

      if (!this.props.attached) {
        switch (this.props.width) {
          case 'thin':
            sliderElement.style.borderTopLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            sliderElement.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            sliderElement.style.borderTopLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            sliderElement.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            sliderElement.style.borderTopLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            sliderElement.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }
      }

      this.item1 = item
      this.item2 = null
    } else if (
      activeIndex > 0 &&
      activeIndex < sidebarElement.children.length - 2
    ) {
      const item1 = sidebarElement.children[activeIndex]
      const item2 = sidebarElement.children[activeIndex + 2]

      switch (this.props.width) {
        case 'thin':
          item1.style.borderBottomRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
          item2.style.borderTopRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
          break
        case 'normal':
          item1.style.borderBottomRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
          item2.style.borderTopRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
          break
        case 'wide':
          item1.style.borderBottomRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
          item2.style.borderTopRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
          break
        default:
          break
      }

      this.item1 = item1
      this.item2 = item2
    } else {
      const item = sidebarElement.children[sidebarElement.children.length - 2]

      switch (this.props.width) {
        case 'thin':
          item.style.borderBottomRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
          break
        case 'normal':
          item.style.borderBottomRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
          break
        case 'wide':
          item.style.borderBottomRightRadius =
            C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
          break
        default:
          break
      }

      if (!this.props.attached) {
        switch (this.props.width) {
          case 'thin':
            sliderElement.style.borderBottomLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            sliderElement.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            sliderElement.style.borderBottomLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            sliderElement.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            sliderElement.style.borderBottomLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            sliderElement.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }
      }

      this.item1 = item
      this.item2 = null
    }

    sliderElement.style.top = `${moveLength}px`
    sliderElement.style.width = `${sliderTo.offsetWidth}px`
    sliderElement.style.height = `${sliderTo.offsetHeight}px`
  }

  effectMoveSlider = prevActiveIndex => {
    const activeIndex = this.state.activeIndex

    this.moving = true

    if (this.item1 !== null) {
      this.item1.style.borderTopRightRadius = ''
      this.item1.style.borderBottomRightRadius = ''
    }
    if (this.item2 !== null) {
      this.item2.style.borderTopRightRadius = ''
      this.item2.style.borderBottomRightRadius = ''
    }

    const sidebarElement = this.sidebar.current
    const sliderElement = this.slider.current

    const sliderTo = sidebarElement.children[activeIndex + 1]
    const sliderTop = parseInt(sliderElement.style.top)
    const minIndex = Math.min(prevActiveIndex, activeIndex)
    const maxIndex = Math.max(prevActiveIndex, activeIndex)
    const moveLength = Array.from(sidebarElement.children)
      .slice(1)
      .slice(minIndex, maxIndex)
      .reduce((acc, child) => {
        return acc + child.offsetHeight
      }, 0)

    const moveChange = () => {
      if (activeIndex === 0) {
        const item = sidebarElement.children[2]
        switch (this.props.width) {
          case 'thin':
            item.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            item.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            item.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }

        this.item1 = item
        this.item2 = null
      } else if (
        activeIndex > 0 &&
        activeIndex < sidebarElement.children.length - 2
      ) {
        const item1 = sidebarElement.children[activeIndex]
        const item2 = sidebarElement.children[activeIndex + 2]

        switch (this.props.width) {
          case 'thin':
            item1.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            item2.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            item1.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            item2.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            item1.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            item2.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }

        this.item1 = item1
        this.item2 = item2
      } else {
        const item = sidebarElement.children[sidebarElement.children.length - 2]

        switch (this.props.width) {
          case 'thin':
            item.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            item.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            item.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }

        this.item1 = item
        this.item2 = null
      }

      this.moving = false
    }

    if (activeIndex === 0) {
      if (!this.props.attached) {
        switch (this.props.width) {
          case 'thin':
            sliderElement.style.borderTopLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            sliderElement.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            sliderElement.style.borderTopLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            sliderElement.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            sliderElement.style.borderTopLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            sliderElement.style.borderTopRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }
      }
    } else if (
      activeIndex > 0 &&
      activeIndex < sidebarElement.children.length - 2
    ) {
      sliderElement.style.borderTopLeftRadius = 0
      sliderElement.style.borderTopRightRadius = 0
      sliderElement.style.borderBottomLeftRadius = 0
      sliderElement.style.borderBottomRightRadius = 0
    } else {
      if (!this.props.attached) {
        switch (this.props.width) {
          case 'thin':
            sliderElement.style.borderBottomLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            sliderElement.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_THIN_RADIUS
            break
          case 'normal':
            sliderElement.style.borderBottomLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            sliderElement.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_NORMAL_RADIUS
            break
          case 'wide':
            sliderElement.style.borderBottomLeftRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            sliderElement.style.borderBottomRightRadius =
              C.CREATIVE_CP_BUBBLE_SIDEBAR_WIDE_RADIUS
            break
          default:
            break
        }
      }
    }

    if (activeIndex > prevActiveIndex) {
      sliderElement.style.top = `${sliderTop + moveLength}px`
    } else {
      sliderElement.style.top = `${sliderTop - moveLength}px`
    }

    sliderElement.style.width = `${sliderTo.offsetWidth}px`
    sliderElement.style.height = `${sliderTo.offsetHeight}px`

    setTimeout(() => {
      this.setState(
        {
          background: 'transparent'
        },
        () => {
          moveChange()
          this.props.onClick && this.props.onClick()
        }
      )
    }, 500)
  }

  render() {
    const { attached, color, width, sliderBackground } = this.props
    const { background } = this.state

    const wrapperClassName = concatClassNamesByCondition([
      { value: 'creative-cp-bubble-sidebar-wrapper', condition: true },
      { value: 'attached', condition: attached },
      { value: width, condition: true }
    ])
    const sidebarClassName = concatClassNamesByCondition([
      { value: 'creative-cp-bubble-sidebar', condition: true },
      { value: width, condition: true }
    ])
    const sliderClassName = concatClassNamesByCondition([
      { value: 'slider', condition: true }
    ])

    const children = React.Children.map(this.props.children, (child, index) => {
      return React.cloneElement(child, {
        index,
        background: this.props.background
      })
    })

    return (
      <div
        className={wrapperClassName}
        style={{ background: sliderBackground }}
      >
        <div
          ref={this.sidebar}
          className={sidebarClassName}
          style={{
            color,
            background
          }}
        >
          <div
            ref={this.slider}
            className={sliderClassName}
            style={{
              background: sliderBackground
            }}
          />
          <BubbleSidebarActiveItemContext.Provider value={this.state}>
            {children}
          </BubbleSidebarActiveItemContext.Provider>
        </div>
      </div>
    )
  }
}

BubbleSidebar.propTypes = {
  attached: PropTypes.bool,
  width: PropTypes.oneOf(['thin', 'normal', 'wide']),
  color: PropTypes.string,
  background: PropTypes.string,
  sliderBackground: PropTypes.string,
  activeIndex: PropTypes.number,
  activeColor: PropTypes.string,
  onClick: PropTypes.func
}

BubbleSidebar.Header = Header
BubbleSidebar.Item = Item
export default BubbleSidebar
